@@ -14,9 +14,9 @@
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['stableinterface'],
-                    'supported_by': 'curated'}
+                    'supported_by': 'certified'}
 
 
 DOCUMENTATION = '''
@@ -55,7 +55,8 @@ options:
     choices: [ "present", "absent", "update" ]
   path:
     description:
-      - When creating or updating, specify the desired path of the resource. If state is present, it will replace the current path to match what is passed in when they do not match.
+      - When creating or updating, specify the desired path of the resource. If state is present,
+        it will replace the current path to match what is passed in when they do not match.
     required: false
     default: "/"
   trust_policy:
@@ -101,7 +102,8 @@ options:
     description:
      - C(always) will update passwords if they differ.  C(on_create) will only set the password for newly created users.
 notes:
-  - 'Currently boto does not support the removal of Managed Policies, the module will error out if your user/group/role has managed policies when you try to do state=absent. They will need to be removed manually.'
+  - 'Currently boto does not support the removal of Managed Policies, the module will error out if your
+    user/group/role has managed policies when you try to do state=absent. They will need to be removed manually.'
 author:
     - "Jonathan I. Davila (@defionscode)"
     - "Paul Seiffert (@seiffert)"
@@ -170,17 +172,6 @@ try:
 except ImportError:
     HAS_BOTO = False
 
-def boto_exception(err):
-    '''generic error message handler'''
-    if hasattr(err, 'error_message'):
-        error = err.error_message
-    elif hasattr(err, 'message'):
-        error = err.message
-    else:
-        error = '%s: %s' % (Exception, err)
-
-    return error
-
 
 def _paginate(func, attr):
     '''
@@ -242,44 +233,63 @@ def create_user(module, iam, name, pwd, path, key_state, key_count):
         return (user_info, changed)
 
 
-def delete_user(module, iam, name):
-    del_meta = ''
+def delete_dependencies_first(module, iam, name):
+    changed = False
+    # try to delete any keys
     try:
         current_keys = [ck['access_key_id'] for ck in
             iam.get_all_access_keys(name).list_access_keys_result.access_key_metadata]
         for key in current_keys:
             iam.delete_access_key(key, name)
-        try:
-            login_profile = iam.get_login_profiles(name).get_login_profile_response
-        except boto.exception.BotoServerError as err:
-            error_msg = boto_exception(err)
-            if ('Cannot find Login Profile') in error_msg:
-                del_meta = iam.delete_user(name).delete_user_response
-        else:
-            iam.delete_login_profile(name)
-            del_meta = iam.delete_user(name).delete_user_response
-    except Exception as ex:
-        module.fail_json(changed=False, msg="delete failed %s" %ex)
-        if ('must detach all policies first') in error_msg:
-            for policy in iam.get_all_user_policies(name).list_user_policies_result.policy_names:
-                iam.delete_user_policy(name, policy)
-            try:
-                del_meta = iam.delete_user(name)
-            except boto.exception.BotoServerError as err:
-                error_msg = boto_exception(err)
-                if ('must detach all policies first') in error_msg:
-                    module.fail_json(changed=changed, msg="All inline polices have been removed. Though it appears"
-                                                          "that %s has Managed Polices. This is not "
-                                                          "currently supported by boto. Please detach the polices "
-                                                          "through the console and try again." % name)
-                else:
-                    module.fail_json(changed=changed, msg=str(error_msg))
-            else:
-                changed = True
-                return del_meta, name, changed
+        changed = True
+    except boto.exception.BotoServerError as err:
+        module.fail_json(changed=changed, msg="Failed to delete keys: %s" % err, exception=traceback.format_exc())
+
+    # try to delete login profiles
+    try:
+        login_profile = iam.get_login_profiles(name).get_login_profile_response
+        iam.delete_login_profile(name)
+        changed = True
+    except boto.exception.BotoServerError as err:
+        error_msg = boto_exception(err)
+        if 'Cannot find Login Profile' not in error_msg:
+            module.fail_json(changed=changed, msg="Failed to delete login profile: %s" % err, exception=traceback.format_exc())
+
+    # try to detach policies
+    try:
+        for policy in iam.get_all_user_policies(name).list_user_policies_result.policy_names:
+            iam.delete_user_policy(name, policy)
+        changed = True
+    except boto.exception.BotoServerError as err:
+        error_msg = boto_exception(err)
+        if 'must detach all policies first' in error_msg:
+            module.fail_json(changed=changed, msg="All inline polices have been removed. Though it appears"
+                                                  "that %s has Managed Polices. This is not "
+                                                  "currently supported by boto. Please detach the polices "
+                                                  "through the console and try again." % name)
+        module.fail_json(changed=changed, msg="Failed to delete policies: %s" % err, exception=traceback.format_exc())
+
+    # try to deactivate associated MFA devices
+    try:
+        mfa_devices = iam.get_all_mfa_devices(name).get('list_mfa_devices_response', {}).get('list_mfa_devices_result', {}).get('mfa_devices', [])
+        for device in mfa_devices:
+            iam.deactivate_mfa_device(name, device['serial_number'])
+        changed = True
+    except boto.exception.BotoServerError as err:
+        module.fail_json(changed=changed, msg="Failed to deactivate associated MFA devices: %s" % err, exception=traceback.format_exc())
+
+    return changed
+
+
+def delete_user(module, iam, name):
+    changed = delete_dependencies_first(module, iam, name)
+    try:
+        iam.delete_user(name)
+    except boto.exception.BotoServerError as ex:
+        module.fail_json(changed=changed, msg="Failed to delete user %s: %s" % (name, ex), exception=traceback.format_exc())
     else:
         changed = True
-        return del_meta, name, changed
+    return name, changed
 
 
 def update_user(module, iam, name, new_name, new_path, key_state, key_count, keys, pwd, updated):
@@ -288,20 +298,18 @@ def update_user(module, iam, name, new_name, new_path, key_state, key_count, key
     if updated and new_name:
         name = new_name
     try:
-        current_keys, status = \
-            [ck['access_key_id'] for ck in
-             iam.get_all_access_keys(name).list_access_keys_result.access_key_metadata],\
-            [ck['status'] for ck in
-                iam.get_all_access_keys(name).list_access_keys_result.access_key_metadata]
+        current_keys = [ck['access_key_id'] for ck in
+                        iam.get_all_access_keys(name).list_access_keys_result.access_key_metadata]
+        status = [ck['status'] for ck in
+                  iam.get_all_access_keys(name).list_access_keys_result.access_key_metadata]
         key_qty = len(current_keys)
     except boto.exception.BotoServerError as err:
         error_msg = boto_exception(err)
         if 'cannot be found' in error_msg and updated:
-            current_keys, status = \
-                [ck['access_key_id'] for ck in
-             iam.get_all_access_keys(new_name).list_access_keys_result.access_key_metadata],\
-                [ck['status'] for ck in
-                iam.get_all_access_keys(new_name).list_access_keys_result.access_key_metadata]
+            current_keys = [ck['access_key_id'] for ck in
+                            iam.get_all_access_keys(new_name).list_access_keys_result.access_key_metadata]
+            status = [ck['status'] for ck in
+                      iam.get_all_access_keys(new_name).list_access_keys_result.access_key_metadata]
             name = new_name
         else:
             module.fail_json(changed=False, msg=str(err))
@@ -343,11 +351,29 @@ def update_user(module, iam, name, new_name, new_path, key_state, key_count, key
                 else:
                     module.fail_json(msg=error_msg)
 
+    try:
+        current_keys = [ck['access_key_id'] for ck in
+                        iam.get_all_access_keys(name).list_access_keys_result.access_key_metadata]
+        status = [ck['status'] for ck in
+                  iam.get_all_access_keys(name).list_access_keys_result.access_key_metadata]
+        key_qty = len(current_keys)
+    except boto.exception.BotoServerError as err:
+        error_msg = boto_exception(err)
+        if 'cannot be found' in error_msg and updated:
+            current_keys = [ck['access_key_id'] for ck in
+                            iam.get_all_access_keys(new_name).list_access_keys_result.access_key_metadata]
+            status = [ck['status'] for ck in
+                      iam.get_all_access_keys(new_name).list_access_keys_result.access_key_metadata]
+            name = new_name
+        else:
+            module.fail_json(changed=False, msg=str(err))
+
+    new_keys = []
     if key_state == 'create':
         try:
             while key_count > key_qty:
-                new_key = iam.create_access_key(
-                    user_name=name).create_access_key_response.create_access_key_result.access_key
+                new_keys.append(iam.create_access_key(
+                    user_name=name).create_access_key_response.create_access_key_result.access_key)
                 key_qty += 1
                 changed = True
 
@@ -356,18 +382,24 @@ def update_user(module, iam, name, new_name, new_path, key_state, key_count, key
 
     if keys and key_state:
         for access_key in keys:
-            if access_key in current_keys:
-                for current_key, current_key_state in zip(current_keys, status):
-                    if key_state != current_key_state.lower():
-                        try:
-                            iam.update_access_key(
-                                access_key, key_state.capitalize(), user_name=name)
-                        except boto.exception.BotoServerError as err:
-                            module.fail_json(changed=False, msg=str(err))
-                        else:
-                            changed = True
+            if key_state in ('active', 'inactive'):
+                if access_key in current_keys:
+                    for current_key, current_key_state in zip(current_keys, status):
+                        if key_state != current_key_state.lower():
+                            try:
+                                iam.update_access_key(access_key, key_state.capitalize(), user_name=name)
+                                changed = True
+                            except boto.exception.BotoServerError as err:
+                                module.fail_json(changed=False, msg=str(err))
+                else:
+                    module.fail_json(msg="Supplied keys not found for %s. "
+                                         "Current keys: %s. "
+                                         "Supplied key(s): %s" %
+                                         (name, current_keys, keys)
+                                     )
 
-                if key_state == 'remove':
+            if key_state == 'remove':
+                if access_key in current_keys:
                     try:
                         iam.delete_access_key(access_key, user_name=name)
                     except boto.exception.BotoServerError as err:
@@ -391,7 +423,7 @@ def update_user(module, iam, name, new_name, new_path, key_state, key_count, key
     for fk, fks in zip(final_keys, final_key_status):
         updated_key_list.update({fk: fks})
 
-    return name_change, updated_key_list, changed
+    return name_change, updated_key_list, changed, new_keys
 
 
 def set_users_groups(module, iam, name, groups, updated=None,
@@ -453,22 +485,24 @@ def delete_group(module=None, iam=None, name=None):
         iam.delete_group(name)
     except boto.exception.BotoServerError as err:
         error_msg = boto_exception(err)
-        if ('must detach all policies first') in error_msg:
+        if ('must delete policies first') in error_msg:
             for policy in iam.get_all_group_policies(name).list_group_policies_result.policy_names:
                 iam.delete_group_policy(name, policy)
             try:
                 iam.delete_group(name)
             except boto.exception.BotoServerError as err:
                 error_msg = boto_exception(err)
-                if ('must detach all policies first') in error_msg:
+                if ('must delete policies first') in error_msg:
                     module.fail_json(changed=changed, msg="All inline polices have been removed. Though it appears"
                                                           "that %s has Managed Polices. This is not "
                                                           "currently supported by boto. Please detach the polices "
                                                           "through the console and try again." % name)
                 else:
-                    module.fail_json(changed=changed, msg=str(err))
+                    module.fail_json(changed=changed, msg=str(error_msg))
             else:
                 changed = True
+        else:
+            module.fail_json(changed=changed, msg=str(error_msg))
     else:
         changed = True
     return changed, name
@@ -508,6 +542,8 @@ def create_role(module, iam, name, path, role_list, prof_list, trust_policy_doc)
                 instance_profile_result = iam.create_instance_profile(name,
                     path=path).create_instance_profile_response.create_instance_profile_result.instance_profile
                 iam.add_role_to_instance_profile(name, name)
+        else:
+            instance_profile_result = iam.get_instance_profile(name).get_instance_profile_response.get_instance_profile_result.instance_profile
     except boto.exception.BotoServerError as err:
         module.fail_json(changed=changed, msg=str(err))
     else:
@@ -696,12 +732,21 @@ def main():
                 password = None
             if name not in orig_user_list and new_name in orig_user_list:
                 been_updated = True
-            name_change, key_list, user_changed = update_user(
+            name_change, key_list, user_changed, new_key = update_user(
                 module, iam, name, new_name, new_path, key_state, key_count, key_ids, password, been_updated)
+            if new_key:
+                user_meta = {'access_keys': list(new_key)}
+                user_meta['access_keys'].extend(
+                    [{'access_key_id': key, 'status': value} for key, value in key_list.items() if
+                     key not in [it['access_key_id'] for it in new_key]])
+            else:
+                user_meta = {
+                    'access_keys': [{'access_key_id': key, 'status': value} for key, value in key_list.items()]}
+
             if name_change and new_name:
                 orig_name = name
                 name = new_name
-            if groups:
+            if isinstance(groups, list):
                 user_groups, groups_changed = set_users_groups(
                     module, iam, name, groups, been_updated, new_name)
                 if groups_changed == user_changed:
@@ -712,19 +757,24 @@ def main():
                 changed = user_changed
             if new_name and new_path:
                 module.exit_json(changed=changed, groups=user_groups, old_user_name=orig_name,
-                                 new_user_name=new_name, old_path=path, new_path=new_path, keys=key_list)
+                                 new_user_name=new_name, old_path=path, new_path=new_path, keys=key_list,
+                                 created_keys=new_key, user_meta=user_meta)
             elif new_name and not new_path and not been_updated:
                 module.exit_json(
-                    changed=changed, groups=user_groups, old_user_name=orig_name, new_user_name=new_name, keys=key_list)
+                    changed=changed, groups=user_groups, old_user_name=orig_name, new_user_name=new_name, keys=key_list,
+                    created_keys=new_key, user_meta=user_meta)
             elif new_name and not new_path and been_updated:
                 module.exit_json(
-                    changed=changed, groups=user_groups, user_name=new_name, keys=key_list, key_state=key_state)
+                    changed=changed, groups=user_groups, user_name=new_name, keys=key_list, key_state=key_state,
+                    created_keys=new_key, user_meta=user_meta)
             elif not new_name and new_path:
                 module.exit_json(
-                    changed=changed, groups=user_groups, user_name=name, old_path=path, new_path=new_path, keys=key_list)
+                    changed=changed, groups=user_groups, user_name=name, old_path=path, new_path=new_path,
+                    keys=key_list, created_keys=new_key, user_meta=user_meta)
             else:
                 module.exit_json(
-                    changed=changed, groups=user_groups, user_name=name, keys=key_list)
+                    changed=changed, groups=user_groups, user_name=name, keys=key_list, created_keys=new_key,
+                    user_meta=user_meta)
 
         elif state == 'update' and not user_exists:
             module.fail_json(
@@ -734,7 +784,7 @@ def main():
             if user_exists:
                 try:
                     set_users_groups(module, iam, name, '')
-                    del_meta, name, changed = delete_user(module, iam, name)
+                    name, changed = delete_user(module, iam, name)
                     module.exit_json(deleted_user=name, changed=changed)
 
                 except Exception as ex:
